import java.io.*;
import com.swath.*;
import com.swath.cmd.*;

/**
 * Probe exploration script for SWATH.  Reads list of target sectors
 * from a text file.  Automatic or interactive probing, automatic
 * restocking of probes when used at Stardock.  In interactive mode,
 * displays the route to each target sector and lets the user decide
 * whether to probe the route or skip to the next target.  The first
 * of a series of scripts to be released with Mongoose's Evil Kit.
 *
 * <p>1.1 CHANGES:<br>
 * Remembers target list filename between runs.  Fixed a minor bug
 * which may not have ever existed.
 *
 * <p>Bug reports to: krum@oco.net
 *
 * @author Mongoose
 * @version 1.1, 12 June 2001
 */
public class ProbeHunter extends UserDefinedScript {

        protected Parameter fileNameP;
        protected Parameter autoP;
        protected static String fileName = "eprobe.txt";
        protected static final String title = "Probe Hunter";
        protected static final String error = "Script Error";

public String getName() { return title; }

public boolean initScript() throws Exception {
        /* check prompt */
        if(!atPrompt(Swath.COMMAND_PROMPT)) {
                MessageBox.exec("You must be at the command prompt.",
                        error,
                        MessageBox.ICON_ERROR,
                        MessageBox.TYPE_OK);
                return false;
        }

        fileNameP = new Parameter("Target file");
        fileNameP.setType(Parameter.STRING);
        fileNameP.setString(fileName);

        autoP = new Parameter("Auto probe?");
        autoP.setType(Parameter.BOOLEAN);
        autoP.setBoolean(false);

        registerParam(fileNameP);
        registerParam(autoP);

        return true;
}

public boolean runScript() throws Exception {
        int target;
        int probed = 0;
        int sectors = Swath.main.sectors();
        boolean auto = autoP.getBoolean();
        BufferedReader targetList;

        /* set up file reader */
        fileName = fileNameP.getString();
        try {
                targetList = new BufferedReader(new FileReader(fileName));
        } catch(FileNotFoundException e) {
                MessageBox.exec("Target file does not exist.",
                        error,
                        MessageBox.ICON_ERROR,
                        MessageBox.TYPE_OK);
                return false;
        }

        /* refresh info, force wait so swath will notice */
        SendString.exec("i");
        WaitForText.exec("Trader Name");
        while(!atPrompt(Swath.COMMAND_PROMPT)) Thread.sleep(200);

        /* if you have no probes, and are either not at stardock or broke */
        if(Swath.ship.etherProbes() == 0 &&
        (Swath.main.currSector() != Swath.main.stardock() ||
        Swath.you.credits() < 3000)) {
                MessageBox.exec("You have no probes.",
                        error,
                        MessageBox.ICON_ERROR,
                        MessageBox.TYPE_OK);
                return false;
        }

        while(targetList.ready()) {
                /* get the next target */
                try { target = Integer.parseInt(targetList.readLine()); }
                catch(NumberFormatException e) { continue; }
                catch(IOException e) {
                        MessageBox.exec("Error reading from target list.",
                                error,

                                MessageBox.ICON_ERROR,
                                MessageBox.TYPE_OK);
                        return false;
                }
                /* validate target */
                if(target <= 0 || target > sectors) continue;
                /* check probes, buy more as needed */
                if(Swath.ship.etherProbes() == 0) {
                        if(Swath.main.currSector() == Swath.main.stardock() &&
                        Swath.you.credits() >= 3000) {
                                if(auto) buyProbes();
                                else if(MessageBox.exec("Out of probes. Buy more?",
                                        title,
                                        MessageBox.ICON_QUESTION,
                                        MessageBox.TYPE_YES_NO) == MessageBox.RES_YES) buyProbes();
                                else return true;
                        } else {
                                MessageBox.exec("Out of probes.",
                                        error,
                                        MessageBox.ICON_ERROR,
                                        MessageBox.TYPE_OK);
                                return false;
                        }
                }
                /* if auto-probing, just shoot */
                if(auto) {
                        LaunchEtherProbe.exec(target);
                        ++probed;
                }
                /* else display route and ask */
                else {

                        SendString.exec("c");
                        WaitForText.exec("Computer command [TL=");
                        SendString.exec("f\r" + target + "\rq");
                        while(!atPrompt(Swath.COMMAND_PROMPT)) Thread.sleep(200);
                        switch(MessageBox.exec("Shoot this path?",
                                title,
                                MessageBox.ICON_QUESTION,
                                MessageBox.TYPE_YES_NO_CANCEL)) {

                                case MessageBox.RES_YES:
                                        LaunchEtherProbe.exec(target);
                                        ++probed;
                                        break;

                                case MessageBox.RES_NO:
                                        break;

                                case MessageBox.RES_CANCEL:
                                        return true;
                        }
                }
        }
        MessageBox.exec(probed + " targets probed.",
                title + " - Finished",
                MessageBox.ICON_INFORMATION,
                MessageBox.TYPE_OK);
        return true;
}

protected void buyProbes() throws Exception {
        SendString.exec("ps");
        WaitForText.exec("<StarDock> Where to? (?=Help)");
        SendString.exec("he");
        String line = WaitForText.exec("How many Probes do you want (Max");
        /* buy max, blast off, and refresh info */
        SendString.exec(line.substring(33, line.indexOf(')')) + "\rqqi");
        /* wait for prompt */
        while(!atPrompt(Swath.COMMAND_PROMPT)) Thread.sleep(200);
}

} /* end class ProbeHunter */
